package com.moonapps.commandroid;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Timer;
import java.util.TimerTask;

import com.moonapps.commandroid.agents.Agent;
import com.moonapps.commandroid.agents.BatteryController;
import com.moonapps.commandroid.agents.ConnectivityController;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.agents.Controller;
import com.moonapps.commandroid.agents.MetricsManager;
import com.moonapps.commandroid.agents.ScreenController;
import com.moonapps.commandroid.networks.WifiSupportManager;
import com.moonapps.commandroid.networks.WifiSupportUnit;
import com.moonapps.commandroid.profiles.Action;
import com.moonapps.commandroid.profiles.ActionFactory;
import com.moonapps.commandroid.profiles.ActionInfo;
import com.moonapps.commandroid.profiles.Profile;
import com.moonapps.commandroid.profiles.ProfileAlarmManager;
import com.moonapps.commandroid.profiles.ProfileManager;
import com.moonapps.commandroid.profiles.ProfileTriggerComparator;
import com.moonapps.commandroid.profiles.Trigger;
import com.moonapps.commandroid.receivers.BatteryReceiver;
import com.moonapps.commandroid.receivers.ScreenReceiver;
import com.moonapps.commandroid.receivers.WifiTimeReceiver;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Binder;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;


public class DataService extends Service{
	
	private final IBinder mBinder = new DataBinder();

	
    private Timer timer = new Timer(); 
	private Timer checktimer= new Timer();
    
	private static long delayTime = 8*60*60*1000; //8 hours
	private static long checkTime = 5*60*1000; //5 minutes
	
    private HashMap<Integer, Controller> controllers;
	
	private ProfileManager profileManager;
	private WifiSupportManager wifiSupportManager;
	
	
	//SPECIAL INTENT
    private BatteryReceiver batteryreceiver;	
	private ScreenReceiver screenReceiver;

	//Number of data since last check.
	private int dataAmount;


	private boolean screenLocked;

	private HashMap<Integer, Double> actualValues = new HashMap<Integer, Double>();
	private WifiActionManager wifiActionManager;


	private boolean interval;


	private WifiTimeReceiver wifiTimeReceiver;




    
	public class DataBinder extends Binder {
		DataService getService() {
	          return DataService.this;
	      }
	  }

	
	@Override
	public IBinder onBind(Intent arg0) { 
		return mBinder;
	}

	
	
	
	
	@Override 
	public void onCreate () { 
      super.onCreate ();  
      
	  this.controllers = new HashMap <Integer,Controller> ();
	  this.controllers.put(Constants.DEST_BATTERYCONTROLLER, new BatteryController(this));
	  this.controllers.put(Constants.DEST_CONNECTCONTROLLER, new ConnectivityController(this));
	  this.controllers.put(Constants.DEST_SCREENCONTROLLER, new ScreenController(this));

      timer.scheduleAtFixedRate(new SaveTimer(this), delayTime, delayTime);
	  checktimer.scheduleAtFixedRate(new CheckTimer(this), checkTime, checkTime);

      File logFile = new File(LogManager.logOutput);
      if (logFile.exists())
    	  logFile.delete();
      
      
	  this.updateStateVariables();
	  this.startBatteryReceiver();
	  this.startScreenReceiver();  
	  
	  this.startProfileManager();
	  this.startWifiSupportManager();
	  
	  this.startProfileAlarms();
	  this.wifiActionManager = new WifiActionManager(this);
	  this.startWifiTimeReceiver();
  
	  
	  LogManager.print(Log.DEBUG, Constants.debugTag, "DataService: Service Created");
	}
	
	
	private void startWifiTimeReceiver() {

        this.wifiTimeReceiver = new WifiTimeReceiver(this.wifiActionManager);
		IntentFilter filter = new IntentFilter(Constants.ACTION_SEND_ALARM);
        registerReceiver(this.wifiTimeReceiver, filter);
	}





	private void startScreenReceiver() {
		IntentFilter filter = new IntentFilter(Intent.ACTION_SCREEN_ON);
        filter.addAction(Intent.ACTION_SCREEN_OFF);
        this.screenReceiver = new ScreenReceiver();
        registerReceiver(this.screenReceiver, filter);		
	}





	//SPECIAL RECEIVER
	private void startBatteryReceiver() {
		  if (this.batteryreceiver==null){
				this.batteryreceiver =new BatteryReceiver(); 
			  	//Special Intent code.
		      	IntentFilter batteryLevelFilter = new IntentFilter(Intent.ACTION_BATTERY_CHANGED);
			  	registerReceiver(this.batteryreceiver, batteryLevelFilter);
			  }  		
	
	}





	private void startProfileManager() {
		  this.profileManager = (ProfileManager) FileManager.getInstance().loadFileObject(this, ProfileManager.path);
		  if (this.profileManager==null) {
			  LogManager.print(Log.ERROR, Constants.debugTag , "DataService: ProfileM not found. Creating...");
			  this.profileManager = new ProfileManager(); 
		  }if (this.profileManager.getProfiles()==null){
			  LogManager.print(Log.INFO, Constants.debugTag  , "DataService: Bad ProfileM. Fixing.");
			  this.profileManager = new ProfileManager();
		  }else{  
			  LogManager.print(Log.INFO, Constants.debugTag  , "DataService: ProfileM found :). Size:"+ this.profileManager.getProfiles().size());
		  }		
		  
	}
	
	
	private void startWifiSupportManager() {
		  this.wifiSupportManager = (WifiSupportManager) FileManager.getInstance().loadFileObject(this, WifiSupportManager.path);
		  if (this.wifiSupportManager==null) {
			  LogManager.print(Log.ERROR, Constants.debugTag , "DataService: WifSupManager not found. Creating...");
			  this.wifiSupportManager = new WifiSupportManager(); 
		  }if (this.wifiSupportManager.getConfigurations()==null){
			  LogManager.print(Log.INFO, Constants.debugTag  , "DataService: Bad WifSupManager. Fixing.");
			  this.wifiSupportManager = new WifiSupportManager();
		  }else{  
			  LogManager.print(Log.INFO, Constants.debugTag  , "DataService: WifSupManager found :). Size:"+ this.wifiSupportManager.getConfigurations().size());
		  }		
		  
	}





	@Override 
	public void onDestroy () { 
	  this.saveData();
	  LogManager.print(Log.DEBUG, Constants.debugTag, "DataService: Saved & destroyed");
	  
	  
	  
	  
	  //SPECIAL RECEIVERS
	  this.unregisterReceiver(this.batteryreceiver);
	  this.unregisterReceiver(this.screenReceiver);
	  this.unregisterReceiver(this.wifiTimeReceiver);
	  
	  this.wifiActionManager.releaseEverything();
	  
	  this.timer.cancel();
	  this.checktimer.cancel();
	  
	  this.cancelProfileAlarms();

	  super.onDestroy (); 	  
	}
	@Override
	public void onLowMemory(){
		LogManager.print(Log.DEBUG, Constants.debugTag,"DataService: Low memory. Saving:");
		this.saveData();
		
	}

	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		int tmp = super.onStartCommand(intent, flags, startId);

		if (intent == null) {
			return tmp;}

		this.dataAmount = this.dataAmount + this.checkIfReceivingData(intent);
		this.checkIfRequestingData(intent);
		this.checkIfAlarmReceived(intent);
		this.checkInterval(intent);
		
		return tmp;
	}
	
	

			
	




	private void checkIfRequestingData(Intent intent) {
		// Process data requests when extra activityrequest is present.
		int isRequest= intent.getIntExtra(Constants.EXTRAKEY_DATAREQUEST,Constants.NO_EXTRA_ERROR);
		int destination= intent.getIntExtra(Constants.EXTRAKEY_DESTINATION_ID,Constants.NO_EXTRA_ERROR);
		int dataType = intent.getIntExtra(Constants.EXTRAKEY_DATATYPE,Constants.NO_EXTRA_ERROR);
		int graphNumber = intent.getIntExtra(Constants.EXTRAKEY_GRAPHNUMBER,Constants.NO_EXTRA_ERROR);

		
		if (isRequest==Constants.BASICDATA_REQUEST && 
				destination != Constants.NO_EXTRA_ERROR &&
				dataType != Constants.NO_EXTRA_ERROR &&
				graphNumber != Constants.NO_EXTRA_ERROR) {
			this.processBasicDataRequest(destination, dataType, graphNumber);
		}		
		if (isRequest == Constants.PROFILES_REQUEST){
			LogManager.print(Log.ERROR, Constants.debugTag,"DataService: Requested Profiles");
			this.sendProfiles();
		}
		
		if (isRequest == Constants.NETWORKS_REQUEST){
			LogManager.print(Log.ERROR, Constants.debugTag,"DataService: Requested Networks");
			this.sendWifiSupport();
		}
	
		
	}

	private int checkIfReceivingData(Intent intent) {
		int destination = intent.getIntExtra(Constants.EXTRAKEY_DESTINATION_ID, Constants.NO_EXTRA_ERROR);
		int data = intent.getIntExtra(Constants.EXTRAKEY_DATATYPE, Constants.NO_EXTRA_ERROR);	
	
		Bundle bundle = intent.getBundleExtra(Constants.BUNDLEKEY);
	
		
		if (data==Constants.EDITED_PROFILES && destination==Constants.DEST_DATASERVICE){
			ArrayList<Profile> tmp = intent.getParcelableArrayListExtra(Constants.EXTRAKEY_PROFILES);
			if (tmp!=null){
				LogManager.print(Log.DEBUG, Constants.debugTag,"DataService: Received profiles with size:"+tmp.size());
				this.profileManager.setProfiles(tmp);
				this.cancelProfileAlarms();
				this.startProfileAlarms();
			}else{
				LogManager.print(Log.DEBUG, Constants.debugTag,"DataService: Received null profiles.");
			}
		}
		
		if (data==Constants.EDITED_NETWORKS && destination==Constants.DEST_DATASERVICE){
			ArrayList<WifiSupportUnit> tmp = intent.getParcelableArrayListExtra(Constants.EXTRAKEY_NETWORKS);
			if (tmp!=null){
				LogManager.print(Log.DEBUG, Constants.debugTag,"DataService: Received wifisupports with size:"+tmp.size());
				this.wifiSupportManager.setConfigurations(tmp);
				
			}else{
				LogManager.print(Log.DEBUG, Constants.debugTag,"DataService: Received null wifisupports.");
			}
		}

		
		
		if (destination == Constants.DEST_SCREENCONTROLLER && bundle!=null){
			this.screenLocked = (bundle.getInt(Constants.EXTRAKEY_SCREEN_LOCK)==0);
			if (this.screenLocked==false){
				this.wifiActionManager.releaseEverything();
			}else
				this.checkProfiles();
			
		}
		
		
		
		if (destination != Constants.NO_EXTRA_ERROR && bundle!=null) {
			return this.processNewData(destination, bundle);
		}
		return 0;
	}


	//TODO: TEST!
	private void checkIfAlarmReceived(Intent intent) {
		
		Bundle bund = intent.getExtras();
		boolean isAlarm =false;
		
		if (bund!=null)
			isAlarm = bund.getBoolean(Constants.ALARM, false);
	
		
		//ALARM INTENT RECEIVED. ONLY ACTIVATES WHEN SCREEN OFF.
		if (isAlarm && this.screenLocked==true){

			LogManager.print(Log.DEBUG, Constants.debugTag, "   DS: Alarm Intent received" + isAlarm);

			//GET PROFILE AND ISSTART AND:
			//EXECUTE this profile personally, if it still exists on the list.
			
			Bundle bundle = intent.getBundleExtra(Constants.BUNDLEKEY);
			  Profile prof = (Profile) bundle.getParcelable("profile");
			boolean isStart = bundle.getBoolean("isStart", false);
			if (this.profileManager.getProfiles().contains(prof)){
				if (isStart  && prof!=null && this.isProfileTriggerable(prof)){
					this.executeActions(prof.getActions());
					LogManager.print(Log.DEBUG, Constants.debugTag, "   DS: Alarm set off. Prof exists & executed.");
				}
			}
			
			//Anyway, check all profiles, those timed and the normal ones.
			this.checkProfiles();
			
					
		}
				
	}











	private void checkProfiles() {
		LogManager.print(Log.DEBUG, Constants.debugTag, "DataService: Checking profiles...");
		
		
		this.updateStateVariables();
	
		ArrayList<Profile> profiles = new ArrayList<Profile>();
		profiles.addAll(this.profileManager.getProfiles());
		
		Collections.sort(profiles, new ProfileTriggerComparator<Profile>());

		
		
		HashMap<Integer, Action> actionsToDo = new HashMap<Integer, Action>();
	
		
		//GATHER ACTIONS FROM TIMED PROFILES. HIGH PREFERENCE.
		for (Profile p: profiles){
			if (p.isActive() && p.isUsingTimer() && 
					p.isTimeTriggerable() && this.isProfileTriggerable(p)){
				LogManager.print(Log.VERBOSE, Constants.debugTag,"----ProfileTimedActivated:"+p.getName());
				this.gatherActions(actionsToDo, p.getActions());
			}

		}
		
		//GATHER ACTIONS FROM NON-TIMED PROFILES. LOW PREFERENCE.
		for (Profile p: profiles){
			if (p.isActive() && !p.isUsingTimer() && this.isProfileTriggerable(p)){
				LogManager.print(Log.VERBOSE, Constants.debugTag,"----ProfileActivated:"+p.getName());
				this.gatherActions(actionsToDo, p.getActions());
				}

		}
		
		LogManager.print(Log.DEBUG, Constants.debugTag, "DataService: Actions To do:"+actionsToDo.size());
		
		
		this.executeActions(actionsToDo.values());
		
		this.dataAmount=0;
	
		
	}





	private void checkInterval(Intent intent) {
		this.interval = intent.getBooleanExtra(Constants.EXTRAKEY_TIMEINTERVAL, false);
	
		LogManager.print(Log.DEBUG, Constants.debugTag, "DataService: Interval:"+interval+" data:"+this.dataAmount+" scr:"+this.screenLocked);
		
		
		//IF THERE IS NEW DATA, AND THE SCREEN IS OFF.
		if (this.interval==true && this.dataAmount>0 && this.screenLocked==true){
			this.checkProfiles();
	
		}
	}





	private void startProfileAlarms() {
	
		for (Profile p: this.profileManager.getProfiles())
			if (p.isUsingTimer()){
				ProfileAlarmManager.createAlarm(this, p, 
						ProfileAlarmManager.getNextTime(p.getMilliDayStart()).getTime(), true);
				ProfileAlarmManager.createAlarm(this, p, 
						ProfileAlarmManager.getNextTime(p.getMilliDayEnd()).getTime(), false);
			}
		
	}





	private void cancelProfileAlarms() {
	
		for (Profile p: this.profileManager.getProfiles())
			if (p.isUsingTimer()){
				ProfileAlarmManager.cancelAlarm(this, p,
						ProfileAlarmManager.getNextTime(p.getMilliDayStart()).getTime(), true);
				ProfileAlarmManager.cancelAlarm(this, p, 
						ProfileAlarmManager.getNextTime(p.getMilliDayEnd()).getTime(), false);
			}
		
	}





	private void gatherActions(HashMap<Integer, Action> actionsToDo,
			ArrayList<Action> actions) {
		for (Action a: actions)
			if (!actionsToDo.containsKey(a.getActionID()))//TODO: CHECK SPECIAL MODES
				actionsToDo.put(a.getActionID(), a);		
	}





	private void executeActions(Collection<Action> values) {
		boolean wifiUsed = false;
		for (Action act: values){
			if (MetricsManager.actions.containsKey(act.getActionID())){
				ActionInfo actInfo = MetricsManager.actions.get(act.getActionID());
				if (act.getActionID()==Constants.ACTION_WIFI)
					wifiUsed=true;
				
				if (actInfo.canExecute()){
					LogManager.print(Log.ERROR, Constants.debugTag, "DataService... Executing Action");
					ActionFactory.executeNormalAction(this, act);
				}else{
					//TODO: SPECIALMODES
					LogManager.print(Log.ERROR, Constants.debugTag, "DataService... Special Mode.");
					this.executeSpecialAction(act);
	
				}
			}else{
				LogManager.print(Log.ERROR, Constants.debugTag, "DataService... Unrecognized Action.");
			}
		}		
		
		if (wifiUsed==false || values.isEmpty()){
			LogManager.print(Log.ERROR, Constants.debugTag, "DataService: No actions, releasing locks & timers Wifiac:"+wifiUsed);
			this.wifiActionManager.releaseEverything();
		}
	}





	private void executeSpecialAction(Action act) {
		// TODO: IF WIFI ACTION, EXECUTE IN WIFIMANAGER.
		int id = 0;
		if (act!=null)
			id = act.getActionID();
		
		switch (id){
		case (Constants.ACTION_WIFI):
			LogManager.print(Log.DEBUG, Constants.debugTag, "***DS: ExecutingSpecialAction. Wifi.");
			this.wifiActionManager.processAction(act);
			break;
		default:
			break;
		
		}
	}





	private boolean isProfileTriggerable(Profile p) {
		if (p==null)
			return false;
		if (p.getTriggers()==null)
			return false;
		if (p.getTriggers().size()==0)
			return true;
		
		int id=Integer.MIN_VALUE;
		int operator = Integer.MIN_VALUE;
		double value = Double.MIN_VALUE;
		double actualValue = Double.MIN_VALUE;
		boolean result = true;
		
		for (Trigger t: p.getTriggers()){
			id = t.getTriggerID();
			operator = t.getOperator();
			value = t.getValue();
			if (this.actualValues!=null && this.actualValues.containsKey(id))
				actualValue = this.actualValues.get(id);
			
			boolean condition = false;
			
			if (MetricsManager.applyOperator(actualValue, operator, value))
				condition=true;
			
			LogManager.print(Log.DEBUG, Constants.debugTag,"***DataService: "+actualValue+"  op:"+operator+" "+value+ "==>"+condition);
			result = result && condition;
		}
			
			
		return result;
	}





	private void updateStateVariables() {
		
		for (Controller e: this.controllers.values())
			for (Agent y: e.agents.values())
				this.actualValues.put(y.getResourceID(), y.getLastValue());
		
	}







	private void sendProfiles() {
		
		//ActivityProfileReceiver in Manifest.
		Intent toSend = new Intent(Constants.ACTION_PROFILESRESPONSE);
		toSend.putParcelableArrayListExtra(Constants.EXTRAKEY_DATA, this.profileManager.getProfiles());
		this.sendBroadcast(toSend);
		LogManager.print(Log.ERROR, Constants.debugTag, "DataService: Sending Intent. Profs Size:"+this.profileManager.getProfiles().size());
		
	}

	private void sendWifiSupport() {
		
		//ActivityProfileReceiver in Manifest.
		Intent toSend = new Intent(Constants.ACTION_NETWORKSRESPONSE);
		toSend.putParcelableArrayListExtra(Constants.EXTRAKEY_DATA, this.wifiSupportManager.getConfigurations());
		this.sendBroadcast(toSend);
		LogManager.print(Log.ERROR, Constants.debugTag, "DataService: Sending Intent. WifSupp Size:"+this.wifiSupportManager.getConfigurations().size());
		
	}



	





	private int processNewData(int destination, Bundle bundle){
		Controller destCont = this.controllers.get(destination);
		int dataAmount = 0;
		
		if (destCont!=null){
			LogManager.print(Log.DEBUG, Constants.debugTag,"DataService: Received extras&intent to:"+ destination);
			dataAmount = destCont.processBundle(bundle);		
		}else{
			LogManager.print(Log.DEBUG, Constants.debugTag,"DataService: Received intent without destination");
		}
			
		return dataAmount;
	}

	private void processBasicDataRequest(int destination, int dataType, int graphNumber) {

		LogManager.print(Log.DEBUG, Constants.debugTag,"DataService: Requested info. Calling...");
		Controller destCont = this.controllers.get(destination);
		if (destCont!=null){
			LogManager.print(Log.DEBUG, Constants.debugTag,"DataService: Received request to:"+ destination);
			destCont.processRequest(this, dataType, graphNumber);
		}
		
		
	}
	
	public void saveData(){
		FileManager.getInstance().saveFileObject(this, ProfileManager.path, this.profileManager);
		FileManager.getInstance().saveFileObject(this, WifiSupportManager.path, this.wifiSupportManager);

		for (Controller e: this.controllers.values())
			e.saveFile(this);
 
	}
	
	
}



class SaveTimer extends TimerTask
{ 
	DataService serv;
	public SaveTimer(DataService serv){
		this.serv=serv;
		
	}
		
    public void run() 
    {
    	LogManager.print(Log.DEBUG, Constants.debugTag,"DataService: Save timer activated.");
    	this.serv.saveData(); 
    	
    }
}    

class CheckTimer extends TimerTask
{ 
	Context ctx;
	public CheckTimer (Context ctx){
		this.ctx=ctx;		
	}
		
    public void run() 
    {
    	LogManager.print(Log.DEBUG, Constants.debugTag,"DataService: Check timer activated.");
    	Intent startServiceIntent = new Intent(this.ctx, DataService.class);
        startServiceIntent.putExtra(Constants.EXTRAKEY_TIMEINTERVAL, true);
        this.ctx.startService(startServiceIntent);    	
    }
}    
