package com.moonapps.commandroid;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import com.moonapps.commandroid.R;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.listeners.SelectNetListener;
import com.moonapps.commandroid.networks.WifiSupportUnit;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;
import android.widget.Toast;

public class EditNetworkActivity extends Activity implements OnClickListener, OnCheckedChangeListener,OnEditorActionListener{

	WifiSupportUnit editingUnit;
	private EditText editName;
	private CheckBox checkStaticIP;
	
	private int editingPosition;
	private RelativeLayout relativeStaticIP;
	private EditText editIP;
	private EditText editNetmask;
	private EditText editGateway;
	private EditText editDNS1;
	private EditText editDNS2;
	private Button buttonSSID;
	//private Button buttonBSSID;
	
	@Override
	    public void onCreate(Bundle savedInstanceState) {
	        super.onCreate(savedInstanceState);
	        this.setContentView(R.layout.editnetwork);
	        
	        
	        //INITIAL CODE: EDITING... TODO: SET ADDBUTTON TEXT?
	        
	        Intent startingIntent = this.getIntent();
	        WifiSupportUnit wifSupp = startingIntent.getParcelableExtra(Constants.EXTRAKEY_NETWORKS);
	        if (wifSupp==null){
		        this.editingUnit = new WifiSupportUnit();
		        this.editingPosition = Constants.NO_EXTRA_ERROR;
	        }else{
	        	this.editingUnit=wifSupp;
	        	this.editingPosition=startingIntent.getIntExtra(Constants.EXTRAKEY_POSITION, Constants.NO_EXTRA_ERROR);
	        }
	        
	    	
	        //EDITTEXT - DO NOT MOVE.
	        this.editName = (EditText)findViewById(R.id.editNetName);
	        this.editIP = (EditText)findViewById(R.id.editIP); 
	        this.editNetmask = (EditText)findViewById(R.id.editNetmask); 
	        this.editGateway = (EditText)findViewById(R.id.editGateway); 
	        this.editDNS1 = (EditText)findViewById(R.id.editDNS1); 
	        this.editDNS2 = (EditText)findViewById(R.id.editDNS2); 
	        
	        
	        //TODO: IS WORKING? KEYBOARD GONE?
	        this.editName.setOnEditorActionListener(this);
	        this.editIP.setOnEditorActionListener(this);
	        this.editNetmask.setOnEditorActionListener(this);
	        this.editGateway.setOnEditorActionListener(this);
	        this.editDNS1.setOnEditorActionListener(this);
	        this.editDNS2.setOnEditorActionListener(this);

	        //CHECKBOX AND TIME - DO NOT MOVE.
	        this.checkStaticIP = (CheckBox)findViewById(R.id.checkStaticEnabled); 
	        this.checkStaticIP.setChecked(this.editingUnit.isStaticIPEnabled());
	        this.checkStaticIP.setOnCheckedChangeListener(this);

	        //RELATIVE STATIC IP
	        this.relativeStaticIP = (RelativeLayout)findViewById(R.id.staticIPConfig);
	        
	        

	        //TODO: CREATE BSSID & SSID BUTTONS AND ASSIGN ONCLICKLISTENERS
	        this.buttonSSID = (Button) this.findViewById(R.id.selectSsid);
	        //this.buttonBSSID = (Button) this.findViewById(R.id.selectBssid);
	        
	        this.buttonSSID.setOnClickListener(new SelectNetListener(
	        		this, this.editingUnit, this.buttonSSID));
	        
	        
	        //TODO: MODIFICAR CUANDO VENGA UN OBJETO. CON OTROS ATRIBUTOS/LISTAS.
	        this.editName.setText(this.editingUnit.getName());
	        this.buttonSSID.setText(this.editingUnit.getSsid());
	        this.editIP.setText(this.editingUnit.getIp());
	        this.editNetmask.setText(this.editingUnit.getNetmask());
	        this.editGateway.setText(this.editingUnit.getGateway());
	        this.editDNS1.setText(this.editingUnit.getDns1());
	        this.editDNS2.setText(this.editingUnit.getDns2());

	        this.changeStaticCheckBox(this.editingUnit.isStaticIPEnabled());
	  

	       Button result = (Button)findViewById(R.id.finishNetwork);
	       result.setOnClickListener(this);
	  }
	  
	  

	@Override
	public void onClick(View v) {
		
		//TODO: AFTER PRESSING ADD BUTTON. GET NETWORK VALUES AND SEND IF OK.
		
		this.editingUnit.setName(this.editName.getText().toString());
		this.editingUnit.setIp(this.editIP.getText().toString());
		this.editingUnit.setNetmask(this.editNetmask.getText().toString());
		this.editingUnit.setGateway(this.editGateway.getText().toString());
		this.editingUnit.setDns1(this.editDNS1.getText().toString());
		this.editingUnit.setDns2(this.editDNS2.getText().toString());
			
		this.editingUnit.setStaticIPEnabled(this.checkStaticIP.isChecked());
		
		
		if (this.isConfigOK()){
		
			Intent intent = new Intent();
			LogManager.print(Log.DEBUG, Constants.debugTag, "EditNetActivity: Sending Intent...");
			Bundle bundle = new Bundle();
			bundle.putSerializable(Constants.EXTRAKEY_NETWORKS, this.editingUnit);
			bundle.putInt(Constants.EXTRAKEY_POSITION, this.editingPosition);
			intent.putExtra("returnKey", bundle);
			
			setResult(RESULT_OK, intent);
			finish();
		}
	
	}



	@Override
	public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
 
		if (buttonView==this.checkStaticIP){
			this.changeStaticCheckBox(isChecked);
		}
	}



	private void changeStaticCheckBox(boolean isChecked) {
		
		this.checkStaticIP.setChecked(isChecked);
		
		
		if (!isChecked){
			this.relativeStaticIP.setVisibility(View.GONE);
		}else{
			this.relativeStaticIP.setVisibility(View.VISIBLE);
		}		
	}



	private boolean isConfigOK() {
		
		if (this.editingUnit==null){
			LogManager.print(Log.ERROR, Constants.debugTag, "EditNetAct: Net null?");
			return false;
		}else if(this.editingUnit.getName()==null){
			LogManager.print(Log.ERROR, Constants.debugTag, "EditNetAct: Network name null");
			return false;
		}else if(this.editingUnit.getName().equals("")){
			Toast.makeText(this, this.getText(R.string.errorNetworkNoName), Toast.LENGTH_SHORT).show();
			return false;
		}else if (!Constants.isValidIP(this.editingUnit.getIp())){
			Toast.makeText(this, this.getText(R.string.errorIPnotcorrect), Toast.LENGTH_SHORT).show();
			return false;
		}else if (!Constants.isValidIP(this.editingUnit.getGateway())){
			Toast.makeText(this, this.getText(R.string.errorGatewaynotcorrect), Toast.LENGTH_SHORT).show();
			return false;
		}else if (!Constants.isValidIP(this.editingUnit.getDns1()) && !(this.editingUnit.getDns1().equals("")) ){
			Toast.makeText(this, this.getText(R.string.errorDNS1notcorrect), Toast.LENGTH_SHORT).show();
			return false;
		}else if (!Constants.isValidIP(this.editingUnit.getDns2()) && !(this.editingUnit.getDns2().equals("")) ){
			Toast.makeText(this, this.getText(R.string.errorDNS2notcorrect), Toast.LENGTH_SHORT).show();
			return false;
		}else if (!Constants.isValidNetmask(this.editingUnit.getNetmask())){
			Toast.makeText(this, this.getText(R.string.errorNetmasknotcorrect), Toast.LENGTH_SHORT).show();
			return false;
		}			
		
		else{ //ES CORRECTO
			}
		
		return true;
	}



	@Override
	public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
		
		if (actionId == EditorInfo.IME_ACTION_GO) {
			// hide virtual keyboard
			InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
			imm.hideSoftInputFromWindow(this.editName.getWindowToken(), 0);
			return true;
		}
		return false;
	}





}



