package com.moonapps.commandroid;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.text.SimpleDateFormat;
import java.util.ArrayList;

import com.moonapps.commandroid.R;
import com.moonapps.commandroid.adapters.ActionAdapter;
import com.moonapps.commandroid.adapters.TriggerAdapter;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.listeners.AddActionListener;
import com.moonapps.commandroid.listeners.AddTriggerListener;
import com.moonapps.commandroid.listeners.MainBarClickListener;
import com.moonapps.commandroid.listeners.TimePickerListener;
import com.moonapps.commandroid.profiles.Profile;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.ViewFlipper;
import android.widget.TextView.OnEditorActionListener;
import android.widget.Toast;

public class EditProfileActivity extends Activity implements OnClickListener, OnCheckedChangeListener,OnEditorActionListener{

	Profile editingProfile;
	private EditText editName;
	private CheckBox checkedTime;
	private TextView textFrom;
	private TextView textTo;
	private Button buttonStartTime;
	private Button buttonEndTime;
	
	
	private int editingPosition;
	
	@Override
	    public void onCreate(Bundle savedInstanceState) {
	        super.onCreate(savedInstanceState);
	        this.setContentView(R.layout.editprofile);
	        
	        
	        //INITIAL CODE: EDITING... TODO: SET ADDBUTTON TEXT?
	        Intent startingIntent = this.getIntent();
	        Profile prof = startingIntent.getParcelableExtra(Constants.EXTRAKEY_PROFILES);
	        if (prof==null){
		        this.editingProfile = new Profile();
		        this.editingPosition = Constants.NO_EXTRA_ERROR;
	        }else{
	        	this.editingProfile=prof;
	        	this.editingPosition=startingIntent.getIntExtra(Constants.EXTRAKEY_POSITION, Constants.NO_EXTRA_ERROR);
	        }
	        
	        //PREPARE MAIN VIEWFLIPPER
	    	this.prepareXMLFlipper();
	    	
	        //EDITTEXT - DO NOT MOVE.
	        this.editName = (EditText)findViewById(R.id.editName); 
	        this.editName.setOnEditorActionListener(this);

	        //CHECKBOX AND TIME - DO NOT MOVE.
	        this.checkedTime = (CheckBox)findViewById(R.id.checkTime); 
	        this.checkedTime.setChecked(this.editingProfile.isUsingTimer());
	        this.checkedTime.setOnCheckedChangeListener(this);

	        this.textFrom = (TextView)findViewById(R.id.from);
	        this.textTo = (TextView)findViewById(R.id.to);
	        this.buttonStartTime = (Button)findViewById(R.id.startingTime);
	        this.buttonEndTime = (Button)findViewById(R.id.endingTime);

	        
	        this.buttonStartTime.setOnClickListener(
	        		new TimePickerListener(this, this.buttonStartTime, 
	        				this.editingProfile, false));
	        this.buttonEndTime.setOnClickListener(
	        		new TimePickerListener(this, this.buttonEndTime, 
	        				this.editingProfile, true));
	        
	        //LISTS OF TRIGGERS AND ACTIONS.
	        this.prepareXMLListAdapters();

	        
	        //TODO: MODIFICAR CUANDO VENGA UN OBJETO. CON OTROS ATRIBUTOS/LISTAS.
	        this.editName.setText(this.editingProfile.getName());
	        this.timeCheckBoxModified(this.editingProfile.isUsingTimer());
	        this.setTimeButtonsText();
	  

	       Button result = (Button)findViewById(R.id.finishProfile);
	       result.setOnClickListener(this);
	  }
	  
	  
	private void setTimeButtonsText() {
		String dateToSet = "";
		SimpleDateFormat sdf;  
     	sdf= new SimpleDateFormat("HH:mm");
     	
     	dateToSet= sdf.format(this.editingProfile.getMilliDayStart());
		this.buttonStartTime.setText(dateToSet);
       
     	dateToSet= sdf.format(this.editingProfile.getMilliDayEnd());
		this.buttonEndTime.setText(dateToSet);
	}


	private void prepareXMLListAdapters() {

		ListView triggerslist = (ListView) this.findViewById(R.id.triggerslist);
		ImageView addtriggerbutton = (ImageView) this.findViewById(R.id.addtriggerbutton);
		TriggerAdapter triggeradapter = new TriggerAdapter(this, 0,
				this.editingProfile.getTriggers());
		triggerslist.setAdapter(triggeradapter);

		AddTriggerListener addtrigglistener = new AddTriggerListener(this,
				triggeradapter, this.editingProfile.getTriggers());
		addtriggerbutton.setOnClickListener((OnClickListener) addtrigglistener);

		
		
		
        ListView actionslist = (ListView)this.findViewById(R.id.actionslist);
        ImageView addactionbutton = (ImageView)this.findViewById(R.id.addactionbutton);
        ActionAdapter actionadapter = new ActionAdapter(this, 0, this.editingProfile.getActions());
        actionslist.setAdapter(actionadapter);
       
        

        
        
        AddActionListener addactionlistener = new AddActionListener (this, actionadapter, this.editingProfile.getActions());
        addactionbutton.setOnClickListener((OnClickListener) addactionlistener);
        
		
	}


	private void prepareXMLFlipper() {
		ViewFlipper viewFlipper = (ViewFlipper) this.findViewById(R.id.viewflipper);

		LinearLayout areaicon1 = (LinearLayout) this.findViewById(R.id.areaicon1);
		LinearLayout areaicon2 = (LinearLayout) this.findViewById(R.id.areaicon2);
		LinearLayout areaicon3 = (LinearLayout) this.findViewById(R.id.areaicon3);

		ArrayList<LinearLayout> lin = new ArrayList<LinearLayout>();
		lin.add(areaicon1); lin.add(areaicon2); lin.add(areaicon3);
		
		
		OnClickListener listener = new MainBarClickListener(
				this.getApplicationContext(), viewFlipper, lin);
		areaicon1.setOnClickListener(listener);
		areaicon2.setOnClickListener(listener);
		areaicon3.setOnClickListener(listener);

	}


	@Override
	public void onClick(View v) {
		
		this.editingProfile.setName(this.editName.getText().toString());
		this.editingProfile.setUsingTimer(this.checkedTime.isChecked());
		
		
		if (this.isProfileOK()){
		
			Intent intent = new Intent();
			LogManager.print(Log.DEBUG, Constants.debugTag, "EditActivity: Sending Intent...");
			Bundle bundle = new Bundle();
			bundle.putSerializable("profile", this.editingProfile);
			bundle.putInt(Constants.EXTRAKEY_POSITION, this.editingPosition);
			intent.putExtra("returnKey", bundle);
			
			setResult(RESULT_OK, intent);
			finish();
		}
	
	}



	@Override
	public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
 
		if (buttonView==this.checkedTime){
			this.timeCheckBoxModified(isChecked);
		}
	}



	private void timeCheckBoxModified(boolean isChecked) {
		
		this.editingProfile.setUsingTimer(isChecked);
		this.buttonEndTime.setEnabled(isChecked);
		this.buttonStartTime.setEnabled(isChecked);
		if (!isChecked){
			this.textFrom.setTextColor(Color.GRAY);
			this.textTo.setTextColor(Color.GRAY);
		}else{
			this.textFrom.setTextColor(Color.WHITE);
			this.textTo.setTextColor(Color.WHITE);
		}		
	}



	private boolean isProfileOK() {
		
		if (this.editingProfile==null){
			LogManager.print(Log.ERROR, Constants.debugTag, "EditProfileAct: Profile null?");
			return false;
		}else if(this.editingProfile.getName()==null){
			LogManager.print(Log.ERROR, Constants.debugTag, "EditProfileAct: Profile null?");
			return false;
		}else if(this.editingProfile.getName().equals("")){
			Toast.makeText(this, this.getText(R.string.errorProfileNoName), Toast.LENGTH_SHORT).show();
			return false;
		}
			
		else if(this.editingProfile.getActions()==null && this.editingProfile.getActions().size()==0){
			Toast.makeText(this, this.getText(R.string.errorProfileNoAction), Toast.LENGTH_SHORT).show();
			return false;
		}
			
		if (this.editingProfile.isUsingTimer()){
			boolean timeOK = this.editingProfile.getMilliDayStart()<this.editingProfile.getMilliDayEnd();
			if (timeOK) return true;
			else{
				Toast.makeText(this, this.getText(R.string.errorWrongTiming), Toast.LENGTH_SHORT).show();
				return false;
			}
		}
		return true;
	}



	@Override
	public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
		
		if (actionId == EditorInfo.IME_ACTION_GO) {
			// hide virtual keyboard
			InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
			imm.hideSoftInputFromWindow(this.editName.getWindowToken(), 0);
			return true;
		}
		return false;
	}





}



