package com.moonapps.commandroid;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.EOFException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;

import com.moonapps.commandroid.R;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.graph.BasicData;

import android.content.Context;
import android.util.Log;


public class FileManager {

	
	private static FileManager instance;
	public static FileManager getInstance(){
	
		if (instance==null)
			instance=new FileManager();
		
		return instance;
	};
	
	//------SAVE/LOAD DATA CODE:----------
	public int loadData(FileInputStream fileInput, ArrayList<BasicData> data){
		
	    ObjectInputStream input = null;
	    int success = -1;
	    data.clear();
	    
	    try {
	        input = new ObjectInputStream(fileInput);
	        
	        BasicData tmp;
	        while ((tmp = (BasicData)input.readObject())!=null){ 
	        	data.add(tmp);  
	        	}
	        success=0;
	        
	    }
	   catch (EOFException ex) { 	success=0;}
	   catch (Exception ex) {       success=-1;   }
	   
	   finally {
	       try {
	          if (input != null)   
	        	  input.close();
	          if (fileInput != null)   
	        	  fileInput.close();
	       }catch (Exception e) {}
	   }

	   return success;
	}
	

	
public int saveData(FileOutputStream fileOutput, ArrayList<BasicData> data){
	ObjectOutputStream output = null;
	int success = -1;
	
	try {
		output = new ObjectOutputStream(fileOutput);
	      
        for (BasicData e : data){
        	output.writeObject(e);
        }
        success=0;
	} catch (Exception e) {	success=-1;
	}finally {
		
		try{
          if (output != null)   
        	  output.close();
          if (fileOutput != null)   
        	  fileOutput.close();
		}catch (Exception e){}
	}
	
	return success;
	}
	
/*
 * STANDARD METHODS. NOT USED.
 * 
*/
public int loadFile(Context ctx, String path, ArrayList<BasicData> data){

	int success=-1;
try {
	success = this.loadData(ctx.openFileInput(path), data);
    } catch (FileNotFoundException e) {
    	LogManager.print(Log.ERROR, Constants.debugTag, ctx.getString(R.string.loadfileerror));
		success=-1;
    }
    return success;
}


public int saveFile(Context ctx, String path, ArrayList<BasicData> data){
		
	try {
		this.saveData(ctx.openFileOutput(path, Context.MODE_PRIVATE), data);
		return 0;
	} catch (FileNotFoundException e) {
		LogManager.print(Log.ERROR, Constants.debugTag,ctx.getString(R.string.savefileerror));
		//TODO: Optional Toast.
		return -1;
	}
}




//------SAVE/LOAD DATA CODE:----------
public Object loadObject(FileInputStream fileInput){
	Object tmp = null;
    ObjectInputStream input = null;	    
    try {
        input = new ObjectInputStream(fileInput);
    
       
        if ((tmp = (Object)input.readObject())!=null){ 
        	
        	return tmp;
        }
        
    }
   catch (EOFException ex) { return tmp;	}
   catch (Exception ex) {     return null;  }
   
   finally {
       try {
          if (input != null)   
        	  input.close();
          if (fileInput != null)   
        	  fileInput.close();
       }catch (Exception e) {}
       
   }
	return tmp;

}



public int saveObject(FileOutputStream fileOutput,Object data){
ObjectOutputStream output = null;
int success = -1;

try {
	output = new ObjectOutputStream(fileOutput);
    output.writeObject(data);
    success=0;
} catch (Exception e) {	success=-1;
}finally {
	
	try{
      if (output != null)   
    	  output.close();
      if (fileOutput != null)   
    	  fileOutput.close();
	}catch (Exception e){}
}

return success;
}


public Object loadFileObject(Context ctx, String path){

	Object tmp=null;
try {
	tmp = this.loadObject(ctx.openFileInput(path));
    } catch (FileNotFoundException e) {
    	LogManager.print(Log.ERROR, Constants.debugTag, ctx.getString(R.string.loadfileerror));
		tmp=null;
    }
    return tmp;
}


public int saveFileObject(Context ctx, String path, Object data){
		
	try {
		this.saveObject(ctx.openFileOutput(path, Context.MODE_PRIVATE), data);
		return 0;
	} catch (FileNotFoundException e) {
		LogManager.print(Log.ERROR, Constants.debugTag,ctx.getString(R.string.savefileerror));
		//TODO: Optional Toast.
		return -1;
	}
}

	
}
