package com.moonapps.commandroid;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.ArrayList;

import android.app.Activity;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.ViewFlipper;

import com.moonapps.commandroid.adapters.ProfileAdapter;
import com.moonapps.commandroid.adapters.WifiSupportAdapter;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.dialogs.ColorDialog;
import com.moonapps.commandroid.dialogs.DataDialog;
import com.moonapps.commandroid.dialogs.PointStyleDialog;
import com.moonapps.commandroid.graph.DataGraph;
import com.moonapps.commandroid.graph.GraphManager;
import com.moonapps.commandroid.listeners.ColorButtonListener;
import com.moonapps.commandroid.listeners.DataButtonListener;
import com.moonapps.commandroid.listeners.GraphButtonListener;
import com.moonapps.commandroid.listeners.MainBarClickListener;
import com.moonapps.commandroid.listeners.PointStyleButtonListener;
import com.moonapps.commandroid.networks.WifiSupportUnit;
import com.moonapps.commandroid.profiles.Profile;
import com.moonapps.commandroid.receivers.ActivityBatteryReceiver;
import com.moonapps.commandroid.receivers.ActivityNetworksReceiver;
import com.moonapps.commandroid.receivers.ActivityProfilesReceiver;
import com.moonapps.commandroid.receivers.ActivityDataReceiver;



public class MainActivity extends Activity implements View.OnClickListener {
	
	
	
	
	//VARIABLES
	private ActivityDataReceiver dataReceiver;
	private GraphManager graphManager;
	private ActivityProfilesReceiver profilesReceiver;
	private ActivityNetworksReceiver networkReceiver;
	private ArrayList<Profile> profiles;
	private ArrayList<WifiSupportUnit> wifiSupports;

	
	//MAIN BUTTONS
	//private ViewFlipper viewFlipper;
	/*private LinearLayout areaicon1;
	private LinearLayout areaicon2;
	private LinearLayout areaicon3;
	private LinearLayout areaicon4;*/

	//PROFILE SECTION
	private ProfileAdapter profileAdapter;
	
	private WifiSupportAdapter wifiSupportAdapter;
	
	//GRAPH SECTION
	private RelativeLayout graphZone;
	private DataGraph data1;
	private DataGraph data2;
	private ActivityBatteryReceiver batteryreceiver;
	private Button addProfileButton;
	private ImageView addNetButton;

	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.setContentView(R.layout.main); 

         
        
        
        //Start Necessary Systems
        this.startDataService();
		this.profiles = new ArrayList<Profile>();
		this.wifiSupports = new ArrayList<WifiSupportUnit>();
		
        //Prepare Empty Graph
        this.initDataGraphs();
        
        //Prepare Views
        this.prepareXMLViews();
        
        
        //Prepare Secondary Systems
        this.graphManager = new GraphManager(this, this.graphZone, 	this.data1, this.data2);
	

		
		
        //Receivers
        this.dataReceiver = new ActivityDataReceiver (this.data1, this.data2);
      	IntentFilter customFilter = new IntentFilter(Constants.ACTION_DATARESPONSE);
	  	this.registerReceiver(this.dataReceiver, customFilter);
	  	
	  	this.profilesReceiver = new ActivityProfilesReceiver(this.profiles, this.profileAdapter);
	  	IntentFilter customFilter2 = new IntentFilter(Constants.ACTION_PROFILESRESPONSE);
	  	this.registerReceiver(this.profilesReceiver, customFilter2);

	  	this.networkReceiver = new ActivityNetworksReceiver(this.wifiSupports, this.wifiSupportAdapter);
	  	IntentFilter customFilter3 = new IntentFilter(Constants.ACTION_NETWORKSRESPONSE);
	  	this.registerReceiver(this.networkReceiver, customFilter3);
	  	
	  	
	  	//DATA FROM SERVICE
		this.requestProfiles();
		this.requestNetworks();


    }
    




	private void initDataGraphs() {
		this.data1 = new DataGraph();
		this.data2 = new DataGraph();

	}



	private void startDataService() {
        Intent startServiceIntent = new Intent(this, DataService.class);
        this.startService(startServiceIntent);		
	}
	

	
	
	//PREPARING VIEWS
	private void prepareXMLViews() {
    	this.prepareXMLMain();
    	this.prepareXMLGeneral();
    	this.prepareXMLProfile();
    	this.prepareXMLNetworks();
    	this.prepareXMLGraph();
    	
    	//TODO: BORRAR.
    	//Button testingButton = (Button) this.findViewById(R.id.testingButton);
    	//testingButton.setOnClickListener(this);
	}
    
    
      
	private void prepareXMLMain() {
	    ViewFlipper viewFlipper = (ViewFlipper) this.findViewById(R.id.viewflipper);
	
	    LinearLayout areaicon1 = (LinearLayout) this.findViewById(R.id.areaicon1);
	    LinearLayout areaicon2 = (LinearLayout) this.findViewById(R.id.areaicon2);
	    LinearLayout areaicon3 = (LinearLayout) this.findViewById(R.id.areaicon3);
	    LinearLayout areaicon4 = (LinearLayout) this.findViewById(R.id.areaicon4);
	    
	    OnClickListener listener = new MainBarClickListener(this.getApplicationContext(), 
	    		viewFlipper,areaicon1, areaicon2, areaicon3, areaicon4);
	    areaicon1.setOnClickListener(listener);
	    areaicon2.setOnClickListener(listener);
	    areaicon3.setOnClickListener(listener);
	    areaicon4.setOnClickListener(listener);		
	}





	private void prepareXMLGeneral() {
		
		TextView health = (TextView) this.findViewById(R.id.valuehealth);
		TextView level = (TextView) this.findViewById(R.id.valuelevel);
		TextView plugged = (TextView) this.findViewById(R.id.valueplugged);
		TextView status = (TextView) this.findViewById(R.id.valuestatus);
		TextView temperature = (TextView) this.findViewById(R.id.valuetemperature);
		ProgressBar bar = (ProgressBar) this.findViewById(R.id.progressBarBattery);
		
		
		this.batteryreceiver=new ActivityBatteryReceiver(this,health,level,plugged,
				status, temperature, bar); 
      	IntentFilter batteryLevelFilter = new IntentFilter(Intent.ACTION_BATTERY_CHANGED);
	  	registerReceiver(this.batteryreceiver, batteryLevelFilter);
	  
	}



	private void prepareXMLProfile() {
		this.profileAdapter = new ProfileAdapter(this, -1, this.profiles);
		ListView profileListView = (ListView) this.findViewById(R.id.profilelist);
		profileListView.setAdapter(this.profileAdapter);
		
		//CREATE A LISTENER IF POSSIBLE.
		this.addProfileButton = (Button) this.findViewById(R.id.addButton);
		addProfileButton.setOnClickListener(this);
	}



	private void prepareXMLNetworks() {
		//CREATE A LISTENER IF POSSIBLE.
		this.addNetButton = (ImageView) this.findViewById(R.id.addnetworkbutton);
		this.addNetButton.setOnClickListener(this);
		
		this.wifiSupportAdapter = new WifiSupportAdapter(this, -1, this.wifiSupports);
		ListView networksListView = (ListView) this.findViewById(R.id.networkslist);
		networksListView.setAdapter(this.wifiSupportAdapter);
		
	}





	private void prepareXMLGraph() {
	
	    //DATA BUTTONS
	    DataDialog dataDialog=new DataDialog(this,0);
	    DataDialog dataDialog2=new DataDialog(this,1);
	    Button dataButton1 = (Button) this.findViewById(R.id.firstlineb1name);
	    Button dataButton2 = (Button) this.findViewById(R.id.secondlineb1name);
	    DataButtonListener dab1 = new DataButtonListener(
	    		dataDialog, this.data1, dataButton1);
	    DataButtonListener dab2 = new DataButtonListener(
	    		dataDialog2, this.data2, dataButton2);
	    dataButton1.setOnClickListener(dab1);
	    dataButton2.setOnClickListener(dab2);
	
	    
	    //COLOR BUTTONS
	    ColorDialog colorDialog=new ColorDialog(this);
	    Button colorButton1 = (Button) this.findViewById(R.id.firstlineb2color);
	    Button colorButton2 = (Button) this.findViewById(R.id.secondlineb2color);
	    ColorButtonListener col1 = new ColorButtonListener(
	    		colorDialog, this.data1, colorButton1);
	    ColorButtonListener col2 = new ColorButtonListener(
	    		colorDialog, this.data2, colorButton2);
	    colorButton1.setOnClickListener(col1);
	    colorButton2.setOnClickListener(col2);
	
	    //STYLE BUTTONS
	    PointStyleDialog styleDialog = new PointStyleDialog(this);
	    Button styleButton1 = (Button) this.findViewById(R.id.firstlineb3style);
	    Button styleButton2 = (Button) this.findViewById(R.id.secondlineb3style);
	    PointStyleButtonListener point1 = new PointStyleButtonListener(
	    		styleDialog, this.data1, styleButton1);
	    PointStyleButtonListener point2 = new PointStyleButtonListener(
	    		styleDialog, this.data2, styleButton2);
	    styleButton1.setOnClickListener(point1);
	    styleButton2.setOnClickListener(point2);
	    
	    this.graphZone = (RelativeLayout) this.findViewById(R.id.uppergraph);  
	
	    Button generateButton = (Button) this.findViewById(R.id.button);
	    GraphButtonListener generateclick = new GraphButtonListener(this,this.graphManager, this.graphZone, 
				this.data1, this.data2);
	    generateButton.setOnClickListener(generateclick);
	    		
	}








	private void requestProfiles() {
    	
	  	Intent requestIntent = new Intent(this, DataService.class);
        requestIntent.putExtra(Constants.EXTRAKEY_DATAREQUEST, Constants.PROFILES_REQUEST);
        this.startService(requestIntent);
        LogManager.print(Log.DEBUG, Constants.debugTag, "Activity: Sending Profile Request");
	}
	
	private void requestNetworks() {
    	
	  	Intent requestIntent = new Intent(this, DataService.class);
        requestIntent.putExtra(Constants.EXTRAKEY_DATAREQUEST, Constants.NETWORKS_REQUEST);
        this.startService(requestIntent);
        LogManager.print(Log.DEBUG, Constants.debugTag, "Activity: Sending Networks Request");
	}
    
    
    private void sendProfilesEdited() {
	  	Intent requestIntent = new Intent(this, DataService.class);
	  	requestIntent.putExtra(Constants.EXTRAKEY_DESTINATION_ID, Constants.DEST_DATASERVICE);
	    requestIntent.putExtra(Constants.EXTRAKEY_DATATYPE, Constants.EDITED_PROFILES);
	    requestIntent.putParcelableArrayListExtra(Constants.EXTRAKEY_PROFILES, this.profiles);
	    this.startService(requestIntent);
		
	}
    
    private void sendNetworksEdited() {
	  	Intent requestIntent = new Intent(this, DataService.class);
	  	requestIntent.putExtra(Constants.EXTRAKEY_DESTINATION_ID, Constants.DEST_DATASERVICE);
	    requestIntent.putExtra(Constants.EXTRAKEY_DATATYPE, Constants.EDITED_NETWORKS);
	    requestIntent.putParcelableArrayListExtra(Constants.EXTRAKEY_NETWORKS, this.wifiSupports);
	    this.startService(requestIntent);
		
	}



	public void onClick(View view){
		
		
		if (view==this.addProfileButton){
			Intent myIntent = new Intent(this, EditProfileActivity.class);
			this.startActivityForResult(myIntent, 0);
		}else if (view==this.addNetButton){
			Intent myIntent = new Intent(this, EditNetworkActivity.class);
			this.startActivityForResult(myIntent, 0);
		}
		
	}



	@Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
    	if (data!=null){
    		
    		Bundle bundle = data.getBundleExtra("returnKey");
    		
    		//TODO: DANGER
    		WifiSupportUnit wifSupp = (WifiSupportUnit) bundle.getSerializable(Constants.EXTRAKEY_NETWORKS);
    		Profile prof = (Profile) bundle.getSerializable("profile");
			int position = bundle.getInt(Constants.EXTRAKEY_POSITION);
    		
    		if (prof!=null){
    			if (position==Constants.NO_EXTRA_ERROR)
    				this.profiles.add(prof);
    			else{
    				this.profiles.remove(position);
    				this.profiles.add(position, prof);
    			}
    			this.profileAdapter.notifyDataSetChanged();
    		
    		}
    		if (wifSupp!=null){
    			if (position==Constants.NO_EXTRA_ERROR)
    				this.wifiSupports.add(wifSupp);
    			else{
    				this.wifiSupports.remove(position);
    				this.wifiSupports.add(position, wifSupp);
    			}
    			this.wifiSupportAdapter.notifyDataSetChanged();
    		}
    	}
    }    


	@Override
    public void onDestroy(){
    	super.onDestroy();
    	this.sendProfilesEdited();
    	this.sendNetworksEdited();
    	unregisterReceiver(this.dataReceiver);
    	unregisterReceiver(this.profilesReceiver);
    	unregisterReceiver(this.batteryreceiver);
    	unregisterReceiver(this.networkReceiver);
    } 
	@Override
	 public void onPause(){
		 super.onPause();
		 this.sendProfilesEdited();
		 this.sendNetworksEdited();
	 }
	
    

}


