package com.moonapps.commandroid.adapters;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.ArrayList;

import com.moonapps.commandroid.R;
import com.moonapps.commandroid.agents.MetricsManager;
import com.moonapps.commandroid.listeners.ActValueContListener;
import com.moonapps.commandroid.listeners.ActValueDiscreteListener;
import com.moonapps.commandroid.listeners.EditActionListener;
import com.moonapps.commandroid.profiles.Action;
import com.moonapps.commandroid.profiles.ActionInfo;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class ActionAdapter extends ArrayAdapter<Action> implements OnClickListener{

	Context context;
	ArrayList<Action> objects;
	
	public ActionAdapter(Context context, int resource, ArrayList<Action> objects) {
		super(context, resource, objects);
		this.context=context;
		this.objects=objects;
		}
 

	@Override
    public View getView(int position, View convertView, ViewGroup parent) {
		return this.getNewView (position, convertView, parent);                		
	
	}


	private View getNewView(int position, View convertView, ViewGroup parent) {

		Action act = this.getItem(position);
		View view = new View(this.context);

		if (act != null) {
			
			int id = act.getActionID();
			ActionInfo chosenAction = MetricsManager.actions.get(id);

			
			LayoutInflater inflater = (LayoutInflater) this.context
					.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

			view = inflater.inflate(R.layout.listelementaction, parent,	false);
			
			
			this.prepareNameButton(view, position, chosenAction);
			this.prepareValueButton(view,chosenAction, act, position);
			this.prepareDeleteButton(view);
	
		}

		return view;
	}



	private void prepareNameButton(View view, int position, ActionInfo chosenAction) {
		// NAME
		TextView actionname = (TextView) view
				.findViewById(R.id.actionname);
		actionname.setOnClickListener(new EditActionListener(
				this.context, position, this, objects));	

		// SET ID TEXT
		int stringID = (int) chosenAction.getStringID();
		String text = (String) this.context.getText(stringID);
		actionname.setText(text);
	}


	private void prepareValueButton(View view, ActionInfo chosenAction, Action action, int position) {

		TextView valueText = (TextView) view.findViewById(R.id.actionvalue);
		if (chosenAction.usesValue()) {
			valueText.setVisibility(View.VISIBLE);
			
			if (chosenAction.isDiscrete()) {
				int value = action.getValue();
				int resourceID = chosenAction.getValues().get(value);
				valueText.setText(resourceID);
			
				valueText.setOnClickListener(new ActValueDiscreteListener(
				  this.context, position, this, this.objects));
				 
			} else {
				
				valueText.setText("" + action.getValue());
				
				valueText.setOnClickListener( new ActValueContListener(this.context,
				 position, this, objects));
				
			}
		} else {
			valueText.setVisibility(View.GONE);
		}

	}


	private void prepareDeleteButton(View view) {
		// DELETEBUTTON
		ImageView deleteactionicon = (ImageView) view.findViewById(R.id.deleteactionicon);
		deleteactionicon.setOnClickListener(this);		
	}


	@Override
	public void onClick(View v) {
		
		
		//PELIGROSO. DEPENDIENTE DE CAMBIOS EN XML.
		
		
		ListView list =  (ListView) v.getParent().getParent();
		RelativeLayout parent = (RelativeLayout) v.getParent();
		int num = list.getChildCount();
		int position=-1;
		int i=0;
		for (i=0;i<num;i++)
			if (parent.equals(list.getChildAt(i)))
				position=i;
		
		if (position!=-1)
			this.remove(this.getItem(position));
		
	}

	
	

	
}
