package com.moonapps.commandroid.agents;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.GregorianCalendar;

import com.moonapps.commandroid.LogManager;

import android.content.Context;
import android.os.BatteryManager;
import android.os.Bundle;
import android.util.Log;


public class BatteryController extends Controller{


	public BatteryController(Context context){
		super();
		this.classname="BatteryController";
		this.initAgents();
		this.loadFile(context);
	}
	
	
	
	
	@Override
	protected void initAgents(){	
	
		this.agents.put(Constants.REQUEST_BATTERY_HEALTH, new BatteryHealthAgent());
		this.agents.put(Constants.REQUEST_BATTERY_LEVEL, new BatteryLevelAgent());
		this.agents.put(Constants.REQUEST_BATTERY_PLUGGED, new BatteryPluggedAgent());
		this.agents.put(Constants.REQUEST_BATTERY_STATUS, new BatteryStatusAgent());
		this.agents.put(Constants.REQUEST_BATTERY_TEMPERATURE, new BatteryTemperatureAgent());

	}

	@Override
	public int processBundle(Bundle bundle){
		int dataAmount = 0;
		
		long tmpTime;
		int tmpHealth, tmpLevel, tmpPlugged, tmpStatus, tmpTemperature;
		
		GregorianCalendar cal = (GregorianCalendar) GregorianCalendar.getInstance();
		tmpTime = cal.getTimeInMillis();

		tmpHealth = bundle.getInt(BatteryManager.EXTRA_HEALTH);
		tmpLevel = bundle.getInt(BatteryManager.EXTRA_LEVEL);
		tmpPlugged = bundle.getInt(BatteryManager.EXTRA_PLUGGED);
		tmpStatus = bundle.getInt(BatteryManager.EXTRA_STATUS);
		tmpTemperature = bundle.getInt(BatteryManager.EXTRA_TEMPERATURE);
		
		
		String log = "";
		for (Agent e : this.agents.values())
			log = log+" "+ e.getData().size()+" ";
		LogManager.print(Log.INFO, Constants.debugTag, this.classname+" Num data"+log);
		
		dataAmount = dataAmount +
		this.agents.get(Constants.REQUEST_BATTERY_HEALTH).processNewData(tmpTime, tmpHealth);
		dataAmount = dataAmount +
		this.agents.get(Constants.REQUEST_BATTERY_LEVEL).processNewData(tmpTime, tmpLevel);
		dataAmount = dataAmount +
		this.agents.get(Constants.REQUEST_BATTERY_PLUGGED).processNewData(tmpTime, tmpPlugged);
		dataAmount = dataAmount +
		this.agents.get(Constants.REQUEST_BATTERY_STATUS).processNewData(tmpTime, tmpStatus);
		dataAmount = dataAmount +
		this.agents.get(Constants.REQUEST_BATTERY_TEMPERATURE).processNewData(tmpTime, tmpTemperature);
		
		log = "";
		for (Agent e : this.agents.values())
			log = log+" "+ e.getData().size()+" ";
		LogManager.print(Log.INFO, Constants.debugTag, this.classname+" Num data processed"+log+" d:"+dataAmount);
		
		return dataAmount;
	}
	
}
