package com.moonapps.commandroid.agents;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.ArrayList;
import java.util.HashMap;

import android.net.ConnectivityManager;
import android.os.BatteryManager;

import com.moonapps.commandroid.R;
import com.moonapps.commandroid.profiles.ActionInfo;

public class MetricsManager {
	
	
	public static HashMap<Integer, Metric> metrics;
	public static HashMap<Integer, ActionInfo> actions;
	

	static {
		metrics = new HashMap<Integer, Metric>();
		actions = new HashMap<Integer, ActionInfo>();

		
		Metric tmp = new Metric();
		HashMap<Integer, Integer> tmpMap = new HashMap<Integer, Integer>();
		
		//BATTERY.........................................
		int controllerID = Constants.DEST_BATTERYCONTROLLER;
		
		//-----------CONTINOUS METRICS-----------
		//Battery Level
		tmp= new Metric(Constants.REQUEST_BATTERY_LEVEL,R.string.battery_level, 
				false, null, 0,50,100,true, controllerID);
	    metrics.put(Constants.REQUEST_BATTERY_LEVEL, tmp);
	    
	    //Battery_temperature
		tmp= new Metric(Constants.REQUEST_BATTERY_TEMPERATURE,R.string.battery_temperature, 
				false, null, -10,20, 100, false, controllerID);
	    metrics.put(Constants.REQUEST_BATTERY_TEMPERATURE, tmp);	    
	    
	    //------------DISCRETE METRICS------------
	    //BATTERY STATUS
	    tmpMap = new HashMap<Integer, Integer>();
	    tmpMap.put(BatteryManager.BATTERY_STATUS_CHARGING, R.string.BATTERY_STATUS_CHARGING);
	    tmpMap.put(BatteryManager.BATTERY_STATUS_DISCHARGING, R.string.BATTERY_STATUS_DISCHARGING);
	    tmpMap.put(BatteryManager.BATTERY_STATUS_FULL, R.string.BATTERY_STATUS_FULL);
	    tmpMap.put(BatteryManager.BATTERY_STATUS_NOT_CHARGING, R.string.BATTERY_STATUS_NOT_CHARGING);
	    tmpMap.put(BatteryManager.BATTERY_STATUS_UNKNOWN, R.string.BATTERY_STATUS_UNKNOWN);
	    tmp= new Metric(Constants.REQUEST_BATTERY_STATUS,R.string.battery_status, 
	    		true, tmpMap, BatteryManager.BATTERY_STATUS_UNKNOWN,
	    		BatteryManager.BATTERY_STATUS_DISCHARGING, 
	    		BatteryManager.BATTERY_STATUS_FULL,true, controllerID);
	    metrics.put(Constants.REQUEST_BATTERY_STATUS, tmp);
	    
	    //BATTERY PLUGGED
	    tmpMap = new HashMap<Integer, Integer>();
	    tmpMap.put(0, R.string.TYPE_NONE);
	    tmpMap.put(BatteryManager.BATTERY_PLUGGED_AC, R.string.BATTERY_PLUGGED_AC);
	    tmpMap.put(BatteryManager.BATTERY_PLUGGED_USB, R.string.BATTERY_PLUGGED_USB);
	    tmp= new Metric(Constants.REQUEST_BATTERY_PLUGGED,R.string.battery_plugged, 
	    		true, tmpMap, 0,0,BatteryManager.BATTERY_PLUGGED_USB, 
	    		false, controllerID);
	    metrics.put(Constants.REQUEST_BATTERY_PLUGGED, tmp);
	    
	    
	    //BATTERY HEALTH 
	    tmpMap = new HashMap<Integer, Integer>();
	    tmpMap.put(BatteryManager.BATTERY_HEALTH_DEAD, R.string.BATTERY_HEALTH_DEAD);
	    tmpMap.put(BatteryManager.BATTERY_HEALTH_GOOD, R.string.BATTERY_HEALTH_GOOD);
	    tmpMap.put(BatteryManager.BATTERY_HEALTH_OVER_VOLTAGE, R.string.BATTERY_HEALTH_OVER_VOLTAGE);
	    tmpMap.put(BatteryManager.BATTERY_HEALTH_OVERHEAT, R.string.BATTERY_HEALTH_OVERHEAT);
	    tmpMap.put(BatteryManager.BATTERY_HEALTH_UNKNOWN, R.string.BATTERY_HEALTH_UNKNOWN);
	    tmpMap.put(BatteryManager.BATTERY_HEALTH_UNSPECIFIED_FAILURE, R.string.BATTERY_HEALTH_UNSPECIFIED_FAILURE);

	    tmp= new Metric(Constants.REQUEST_BATTERY_HEALTH,R.string.battery_health,
	    		true, tmpMap, BatteryManager.BATTERY_HEALTH_UNKNOWN,
	    		BatteryManager.BATTERY_HEALTH_GOOD,BatteryManager.BATTERY_HEALTH_UNSPECIFIED_FAILURE,
	    		false, controllerID);
	    metrics.put(Constants.REQUEST_BATTERY_HEALTH, tmp);
	    
	    
	    
	    //CONNECTIVITYMANAGER.....................................
	    controllerID = Constants.DEST_CONNECTCONTROLLER;
	    
	    
	    //CONNECTION TYPE
	    tmpMap = new HashMap<Integer, Integer>();
	    tmpMap.put(-1, R.string.TYPE_NONE);
	    tmpMap.put(ConnectivityManager.TYPE_MOBILE, R.string.TYPE_MOBILE);
	    tmpMap.put(ConnectivityManager.TYPE_WIFI, R.string.TYPE_WIFI);
	    tmp= new Metric(Constants.REQUEST_CONN_TYPE,R.string.connection_type, 
	    		true, tmpMap, -1, -1,ConnectivityManager.TYPE_WIFI, false, controllerID);
	    metrics.put(Constants.REQUEST_CONN_TYPE, tmp);

	    
	    
	    
	    //SCREENCONTROLLER.............
	    controllerID = Constants.DEST_SCREENCONTROLLER;

	    //SCREEN LOCK
	    tmpMap = new HashMap<Integer, Integer>();
	    tmpMap.put(0, R.string.disabled);
	    tmpMap.put(1, R.string.enabled);
	    tmp= new Metric(Constants.REQUEST_SCREEN_LOCK,R.string.screen_lock, 
	    		true, tmpMap, 0, 0,1, false, controllerID);
	    metrics.put(Constants.REQUEST_SCREEN_LOCK, tmp);
	    
	    
	    
	    
	    
	    //-------------------------------------
	   //---------------ACTIONS------------------ 
	    
	    ActionInfo acted = null;
	   
	    HashMap<Integer, Integer> temporary = new HashMap<Integer, Integer>();
	    temporary.put(0, R.string.disabled);
	    temporary.put(1, R.string.enabled);
	    acted = new ActionInfo(Constants.ACTION_VIBRATE_MODE,R.string.ACTION_VIBRATE_MODE,
	    		true, true, true, temporary,0,0,1 );
	    actions.put(Constants.ACTION_VIBRATE_MODE, acted);
	  
	    acted = new ActionInfo(Constants.ACTION_VOLUME_ALARM,R.string.ACTION_VOLUME_ALARM,
	    		true, true, false, null, 0,50,100 );
	    actions.put(Constants.ACTION_VOLUME_ALARM, acted);
	    
	    acted = new ActionInfo(Constants.ACTION_VOLUME_MEDIA,R.string.ACTION_VOLUME_MEDIA,
	    		true, true, false, null,0,50,100  );
	    actions.put(Constants.ACTION_VOLUME_MEDIA, acted);
	    
	    acted = new ActionInfo(Constants.ACTION_VOLUME_RINGTONE,R.string.ACTION_VOLUME_RINGTONE,
	    		true, true, false, null,0,50,100  );
	    actions.put(Constants.ACTION_VOLUME_RINGTONE, acted);
	    

	    
	    //AIRPLANE MODE
	    temporary = new HashMap<Integer, Integer>();
	    temporary.put(0, R.string.disabled);
	    temporary.put(1, R.string.enabled);
	    acted = new ActionInfo(Constants.ACTION_AIRPLANE_MODE,R.string.ACTION_AIRPLANE_MODE,
	    		true, true, true, temporary,0,0,1  );
	    actions.put(Constants.ACTION_AIRPLANE_MODE, acted);
	  
	    //WIFI MODES
	    temporary = new HashMap<Integer, Integer>();
	    temporary.put(0, R.string.disabled);
	    temporary.put(1, R.string.enabled);
	    temporary.put(2, R.string.wifi_keep_on_normal);
	    temporary.put(3, R.string.wifi_keep_on_full);
	    temporary.put(10, R.string.wifi_repeating_10);
	    temporary.put(30, R.string.wifi_repeating_30);
	    temporary.put(60, R.string.wifi_repeating_60);
	    acted = new ActionInfo(Constants.ACTION_WIFI,R.string.ACTION_WIFI,
	    		false, true, true, temporary,0,0,60  );
	    actions.put(Constants.ACTION_WIFI, acted);
	    
	}
	
	
	
	
	static public boolean applyOperator(Number num1, int operator, Number num2){
		
		switch (operator){
		case Constants.operatorLess: return num1.doubleValue()<num2.doubleValue();
		case Constants.operatorLessEqual: return num1.doubleValue()<=num2.doubleValue();
		case Constants.operatorEqual: return num1.doubleValue()==num2.doubleValue();
		case Constants.operatorGreaterEqual: return num1.doubleValue()>=num2.doubleValue();
		case Constants.operatorGreater: return num1.doubleValue()>num2.doubleValue();
		default: return false;
		}

	}
	static public String getOperatorString(int operator){
		
		switch (operator){
		case Constants.operatorLess: return "<";
		case Constants.operatorLessEqual: return "<=";
		case Constants.operatorEqual: return "==";
		case Constants.operatorGreaterEqual: return ">=";
		case Constants.operatorGreater: return ">";
		default: return "";
		}

	}
	public static ArrayList<Metric> getTriggers (){
		ArrayList<Metric> result = new ArrayList<Metric>();
		
		for (Metric e:  MetricsManager.metrics.values())
			if (e.isTrigger())
				result.add(e);
		
		return result;
	}
	
	
	
	
	
	/*public static double getMaxValue(int id) {

		switch (id) {
		case (Constants.REQUEST_BATTERY_HEALTH):
			return 7.0;
		case (Constants.REQUEST_BATTERY_LEVEL):
			return 100.0;
		case (Constants.REQUEST_BATTERY_PLUGGED):
			return 2.0;
		case (Constants.REQUEST_BATTERY_STATUS):
			return 5.0;
		case (Constants.REQUEST_BATTERY_TEMPERATURE):
			return 90.0;

		case (Constants.REQUEST_CONN_SUBTYPE):
			return 9.0;
		case (Constants.REQUEST_CONN_TYPE):
			return 9.0;
		default:
			return -1.0;
		}
	}*/
/*
	public static double getMinValue(int id) {

		switch (id) {
		case (Constants.REQUEST_BATTERY_HEALTH):
			return 1.0;
		case (Constants.REQUEST_BATTERY_LEVEL):
			return 0.0;
		case (Constants.REQUEST_BATTERY_PLUGGED):
			return 0.0;
		case (Constants.REQUEST_BATTERY_STATUS):
			return 1.0;
		case (Constants.REQUEST_BATTERY_TEMPERATURE):
			return -10.0;

		case (Constants.REQUEST_CONN_SUBTYPE):
			return -1.0;
		case (Constants.REQUEST_CONN_TYPE):
			return -1.0;
		default:
			return -1.0;
		}
	}
*/
	/*public static boolean isDiscrete(int id) {

		switch (id) {
		case (Constants.REQUEST_BATTERY_HEALTH):
			return true;
		case (Constants.REQUEST_BATTERY_LEVEL):
			return false;
		case (Constants.REQUEST_BATTERY_PLUGGED):
			return true;
		case (Constants.REQUEST_BATTERY_STATUS):
			return true;
		case (Constants.REQUEST_BATTERY_TEMPERATURE):
			return false;

		case (Constants.REQUEST_CONN_SUBTYPE):
			return true;
		case (Constants.REQUEST_CONN_TYPE):
			return true;
		case (Constants.REQUEST_SCREEN_LOCK):
			return true;
		default:
			return false;
		}
	}
*/

	
}
