package com.moonapps.commandroid.dialogs;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.ArrayList;

import com.moonapps.commandroid.R;
import com.moonapps.commandroid.adapters.TriggerAdapter;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.agents.Metric;
import com.moonapps.commandroid.agents.MetricsManager;
import com.moonapps.commandroid.profiles.Trigger;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;

public class OperatorDialog extends AlertDialog.Builder implements OnClickListener{

	private int position;
	private ArrayList<Trigger> triggers;
	private TriggerAdapter adapter;
	private Context context;

	public OperatorDialog(Context ctx, int position, ArrayList<Trigger> triggers,
			TriggerAdapter adapter) {
		super(ctx);
		this.context=ctx;
		this.position=position;
		this.triggers=triggers;
		this.adapter=adapter;

  		this.setTitle(R.string.dialog_selectoperator);
		this.setItems(this.getItemsSequence(), (android.content.DialogInterface.OnClickListener) this);
	
	}

	private CharSequence[] getItemsSequence() {
		Trigger pressed = this.triggers.get(this.position);
		Metric metricChosen = MetricsManager.metrics.get(pressed.getTriggerID());
		CharSequence[] result = new CharSequence[0];
		if (!metricChosen.isDiscrete()){
			result=new CharSequence[5];
			result[0]="<";
			result[1]="<=";
			result[2]="=";
			result[3]=">=";
			result[4]=">";
				
		}
		return result;
	}

	@Override
	public void onClick(DialogInterface dialog, int which) {

		Trigger pressed = this.triggers.get(this.position);
		Metric metricChosen = MetricsManager.metrics.get(pressed.getTriggerID());
		if (!metricChosen.isDiscrete()){
			pressed.setOperator(Constants.operatorLess+which);
		
		}else{
			pressed.setOperator(Constants.operatorEqual);
			
		}
		this.adapter.notifyDataSetChanged();
	}

	
	


}
