package com.moonapps.commandroid.dialogs;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.ArrayList;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiManager;
import android.util.Log;
import android.widget.Button;

import com.moonapps.commandroid.LogManager;
import com.moonapps.commandroid.R;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.networks.WifiSupportUnit;

public class SelectNetworkDialog extends AlertDialog.Builder implements OnClickListener {

	private Context context;
	private ArrayList<Integer> networkIDs; 
	private WifiSupportUnit wifiSupport;
	private Button networkButton;
 
	public SelectNetworkDialog(Context ctx, WifiSupportUnit wifiSupport, Button network){
		super(ctx);
		this.context=ctx;
		this.networkButton = network;
		this.wifiSupport = wifiSupport;
		this.networkIDs = new ArrayList<Integer>();
  		this.setTitle(R.string.dialog_selectnetwork);
		this.setItems(this.getItemsSequence(), (android.content.DialogInterface.OnClickListener) this);
	}

	private String[] getItemsSequence() {

		ArrayList<String> strings = new ArrayList<String>();
		String[] sol = new String[0];
		
		WifiManager wifiManager = (WifiManager) this.context.getSystemService(Context.WIFI_SERVICE);
		
		
		LogManager.print(Log.VERBOSE, Constants.debugTag, "SELNET: CONFIGS SIZE: "+wifiManager.getConfiguredNetworks().size());
		
		for (WifiConfiguration config: wifiManager.getConfiguredNetworks()){
			networkIDs.add(config.networkId);
			strings.add(config.SSID);
		}
		return (strings.toArray(sol));
	}

	
	//WHEN OPTION CHOSEN
	@Override
	public void onClick(DialogInterface dialog, int which) {
		
		int networkID = this.networkIDs.get(which);
		
		WifiManager wifiManager = (WifiManager) this.context.getSystemService(Context.WIFI_SERVICE);
		WifiConfiguration configuration = null;

		for (WifiConfiguration e :wifiManager.getConfiguredNetworks()){
			if (e.networkId==networkID){
				configuration = e;
				break;
				}
		}
		
		if (configuration != null){
			this.wifiSupport.setSsid(configuration.SSID);
			this.networkButton.setText(configuration.SSID);
			
		}
		
		
		}

}
