package com.moonapps.commandroid.dialogs;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.ArrayList;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.util.Log;

import com.moonapps.commandroid.LogManager;
import com.moonapps.commandroid.R;
import com.moonapps.commandroid.adapters.TriggerAdapter;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.agents.Metric;
import com.moonapps.commandroid.agents.MetricsManager;
import com.moonapps.commandroid.profiles.Trigger;

public class ValueDiscreteDialog extends AlertDialog.Builder implements OnClickListener {

	private Context context;
	private TriggerAdapter adapter;
	private int position;
	private ArrayList<Trigger> triggers;
	
	public ValueDiscreteDialog(Context ctx, int position, TriggerAdapter adapter,
			ArrayList<Trigger> triggers){
		super(ctx);
		this.context=ctx;
		this.position=position;
		this.triggers=triggers;
		this.adapter=adapter;
		
  		this.setTitle(R.string.dialog_selectvalue);
		this.setItems(this.getItemsSequence(), (android.content.DialogInterface.OnClickListener) this);
	}

	private String[] getItemsSequence() {

		ArrayList<String> result = new ArrayList<String>();
		String[] sol = new String[0];
		
		Trigger selected = this.triggers.get(position);
		Metric metric = MetricsManager.metrics.get(selected.getTriggerID());
		
		
		for (Integer e: metric.getValues().values()){
			result.add((String) this.context.getText(e));
		}
		
		return (result.toArray(sol));
	}

	
	//WHEN OPTION CHOSEN
	@Override
	public void onClick(DialogInterface dialog, int which) {
		
		Trigger selected = this.triggers.get(position);
		Metric metric = MetricsManager.metrics.get(selected.getTriggerID());
		
		int total = metric.getValues().size();
		Integer[] result = new Integer[total]; 
		result = metric.getValues().keySet().toArray(new Integer[total]);
		int value = result[which];
		
		
		if (value<metric.getMinValue() || value> metric.getMaxValue()){
			selected.setValue((int) metric.getStdValue());
		}else{
			selected.setValue(value);
		}
		LogManager.print(Log.DEBUG, Constants.debugTag, "Value"+value);
		
		this.adapter.notifyDataSetChanged();
		
		
		}
}
