package com.moonapps.commandroid.graph;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.achartengine.ChartFactory;
import org.achartengine.GraphicalView;
import org.achartengine.chart.PointStyle;
import org.achartengine.model.XYMultipleSeriesDataset;
import org.achartengine.renderer.SimpleSeriesRenderer;
import org.achartengine.renderer.XYMultipleSeriesRenderer;
import org.achartengine.renderer.XYSeriesRenderer;

import com.moonapps.commandroid.LogManager;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.agents.Metric;
import com.moonapps.commandroid.agents.MetricsManager;

import android.content.Context;
import android.graphics.Color;
import android.graphics.Paint.Align;
import android.util.Log;
import android.widget.RelativeLayout;

public class GraphManager {

	private static final String DATE_FORMAT = "d MMM HH:mm";
	private static final int margin = 20;
	private static final int axisTitleTextSize = 14;  //time y value
	private static final int chartTitleTextSize = 14; // datos
	private static final int labelsTextSize = 12;     // ejes
	private static final int legendTextSize = 12;     //nombre dato
	private static final int xlabelsSize = 4;
	private static final int ylabelsSize = 4;
	private static final int chartValuesTextSize = 12;
	
	
	private XYMultipleSeriesRenderer render;
	private XYMultipleSeriesDataset dataset;
	
	private Context ctx;
	private RelativeLayout container;

	private DataGraph data1;
	private DataGraph data2;
	
	private int axesColor = Color.WHITE;
	private int labelsColor = Color.GRAY;
	
	public GraphManager(Context ctx, RelativeLayout contr, 
							DataGraph data1,DataGraph data2 ){
		this.ctx=ctx;
		this.container = contr;
		
		this.setData1(data1);		
		this.setData2(data2);

		
		this.dataset = new XYMultipleSeriesDataset();
		this.render = new XYMultipleSeriesRenderer(2);
		
	}
	
	private void setSTDGrid(){
		this.render.setAxisTitleTextSize(Constants.dipTOpx(ctx,GraphManager.axisTitleTextSize));
		this.render.setChartTitleTextSize(Constants.dipTOpx(ctx,GraphManager.chartTitleTextSize));
		this.render.setLabelsTextSize(Constants.dipTOpx(ctx,GraphManager.labelsTextSize));
		this.render.setLegendTextSize(Constants.dipTOpx(ctx,GraphManager.legendTextSize));
		this.render.setPointSize(4f);
		this.render.setMargins(new int[] { Constants.dipTOpx(ctx,GraphManager.margin),
				Constants.dipTOpx(ctx,GraphManager.margin), 
				Constants.dipTOpx(ctx,GraphManager.margin), 
				Constants.dipTOpx(ctx,GraphManager.margin) });	
	}
	
	
	private void setSTDGridColors (){
		this.render.setAxesColor(this.axesColor);
		this.render.setLabelsColor(this.labelsColor);
	}
	private void setSTDTitles (){
			String title = "Datos";
			String xTitle = "Time";
			String yTitle= "Value";
			this.render.setChartTitle(title);
			this.render.setXTitle(xTitle);
			this.render.setYTitle(yTitle);
			

			
			this.render.setXLabels(Constants.dipTOpx(ctx,GraphManager.xlabelsSize));
			this.render.setYLabels(Constants.dipTOpx(ctx,GraphManager.ylabelsSize));		
	}

	public void setGridColors (int axisColor, int labelsColor ){
		this.render.setAxesColor(axisColor);
		this.render.setLabelsColor(labelsColor);
	}
	
	
	public void setTitles (String title, String titleX, String titleY){
		this.render.setChartTitle(title);
		this.render.setXTitle(titleX);
		this.render.setYTitle(titleY);
		this.render.setXLabels(Constants.dipTOpx(ctx,GraphManager.xlabelsSize));
		this.render.setYLabels(Constants.dipTOpx(ctx,GraphManager.ylabelsSize));		
}
	
	private void emptyDataSet(){
		int capacity = this.dataset.getSeriesCount();
		
		for (int i=0 ;i<capacity;i++)
			this.dataset.removeSeries(0);
	}
	
	private void emptyRenderStyles(){
		int capacity = this.render.getSeriesRendererCount();

		for (int i=0 ;i<capacity;i++)
			this.render.removeSeriesRenderer(this.render.getSeriesRendererAt(0));

	}
	
	private void addDataSeries(String name, DataGraph data, int pos) {

		int color = data.getColor();
		PointStyle style = data.getStyle(); 
		List<BasicData> results = data.getData();
		
		
		
		
		TimeSeries2 series = new TimeSeries2(name, pos);
		for (BasicData result : results) {
				if (data.getId()==Constants.REQUEST_BATTERY_TEMPERATURE)
					series.add(result.getTime(), (int)(result.getValue()/10));
				else
					series.add(result.getTime(), result.getValue());
		}
		LogManager.print(Log.ERROR, Constants.debugTag, "GM: Trying to paint "+ series.getItemCount());
		
		if (this.dataset.getSeriesCount()>pos)
			this.dataset.removeSeries(pos);
		this.dataset.addSeries(pos, series); 
		
		
		XYSeriesRenderer r = new XYSeriesRenderer();
		r.setColor(color);
		r.setPointStyle(style);
		r.setLineWidth(2f);
		r.setChartValuesTextSize(Constants.dipTOpx(ctx,GraphManager.chartValuesTextSize));
		
		if (this.render.getSeriesRendererCount()>pos)
			this.render.removeSeriesRenderer(this.render.getSeriesRendererAt(pos));
		this.render.addSeriesRenderer(pos,r);
	}
	
	
	private void setMinMaxFromData (List<BasicData> results ){
		
		if (results.size()>0){
			//double xMin = results.get(0).getTime();
			//this.render.setXAxisMin(xMin);

			double xMax =  results.get(results.size() - 1).getTime();
			
			//TODO: CAREFUL. SCALE!! SELECT MAX MIN FROM DATA
			this.render.setXAxisMax(xMax,0);
			this.render.setXAxisMin(xMax -24*60*60*1000,0);
			this.render.setXAxisMax(xMax,1);
			this.render.setXAxisMin(xMax -24*60*60*1000,1);
			
			
			Metric met1 = MetricsManager.metrics.get(this.data1.getId());
			if (met1!=null){
				double yMax0 = met1.getMaxValue();
				double yMin0 =met1.getMinValue();
				if (yMax0!=-1) this.render.setYAxisMax(yMax0, 0);
				if (yMin0!=-1) this.render.setYAxisMin(yMin0, 0);
			}
			
			
			Metric met2 = MetricsManager.metrics.get(this.data2.getId());
			if (met2!=null){
				double yMax1 = met2.getMaxValue();
				double yMin1  = met2.getMinValue();
				if (yMax1!=-1) this.render.setYAxisMax(yMax1, 1);
				if (yMin1!=-1) this.render.setYAxisMin(yMin1, 1);
			}


			/*double yMax0 = MetricsManager.getMaxValue(this.data1.getId());
			double yMin0 = MetricsManager.getMinValue(this.data1.getId());
			double yMax1 = MetricsManager.getMaxValue(this.data2.getId());
			double yMin1 = MetricsManager.getMinValue(this.data2.getId());
			*/
			
			/*BasicData max = Collections.max(results, new ValueComparator<BasicData>());
		BasicData min = Collections.min(results, new ValueComparator<BasicData>());
		double yMin = min.getValue();  
		double yMax = max.getValue();
		this.render.setYAxisMin(yMin);
		this.render.setYAxisMax(yMax);*/
		}
	}
	
	
	private void setDisplayChartValues(boolean bool){
		int num =  this.render.getSeriesRendererCount();
		for (int i=0;i<num;i++){
			SimpleSeriesRenderer seriesRenderer = this.render.getSeriesRendererAt(i);
			seriesRenderer.setDisplayChartValues(bool);
		}
	}
	
	
	private GraphicalView createView() {

		return ChartFactory.getTimeChartView(this.ctx,
			this.dataset, this.render, GraphManager.DATE_FORMAT);

	}
	
	
	private boolean isValidData(DataGraph data){
		if (data==null)
			return false;
		else
			return (data.getData().size()>0);
	}
	
	private String getContextIDString(DataGraph data){
		String axisTitle = "";
		if (data.getId()!=-1){
			Metric chosen = MetricsManager.metrics.get(data.getId());
			axisTitle = (String) this.ctx.getText(chosen.getStringID());
			//axisTitle = Constants.getStringFromID(this.ctx, data.getId());
		}
		return axisTitle;
	}
	
	private void getAxisReady(DataGraph data, int pos){
	
			
			
		this.addDataSeries(this.getContextIDString(data), data, pos);
		this.setMinMaxFromData(data.getData());
		this.render.setYLabelsColor(pos, data.getColor());
		
		
		if (pos==0)
			this.render.setYAxisAlign(Align.LEFT,pos);
		else
			this.render.setYAxisAlign(Align.RIGHT,pos);
	
	}

	private GraphicalView getView(Context context,String title) {
		
		this.setSTDTitles();
		
		this.emptyDataSet();
		this.emptyRenderStyles();
		
		//TODO:
		//this.render.clearYTextLabels();
		
		if (this.isValidData(this.data1)){
			this.getAxisReady(this.data1, 0);
		}
		
		if (this.isValidData(this.data2) && this.isValidData(this.data1)){
			this.getAxisReady(this.data2, 1);

		}
		
		if (!this.isValidData(this.data1)){
			if (this.isValidData(this.data2)){
				this.getAxisReady(this.data2, 0);
			}
		}
		
			
		this.setSTDGrid();
		this.setSTDGridColors();
		this.setDisplayChartValues(true);
		this.setGridColors(Color.GRAY,Color.LTGRAY);
		
		
		//TODO: DANGER!
		if (this.data1!=null)
			this.setAxisIfDiscrete(this.data1,0);
		if (this.data2!=null)
			this.setAxisIfDiscrete(this.data2,1);
		
		
		GraphicalView finish =  this.createView();
		finish.repaint();
		return finish;

	}
  
	

	private void setAxisIfDiscrete(DataGraph data, int pos) {

		if (data.getId()!=0){
		Metric selected = MetricsManager.metrics.get(data.getId());
		LogManager.print(Log.ERROR,Constants.debugTag, "GRAPHERROR: ID: "+data.getId()+" "+selected.getId()+" "+this.ctx.getText(selected.getStringID()));
		
		HashMap<Integer, Integer> values = new HashMap<Integer, Integer>();

		if (selected.isDiscrete() && selected!=null){

			this.render.clearYTextLabels();
			values = selected.getValues();
			Set<Integer> tmp = values.keySet();
			Iterator<Integer> it = tmp.iterator();
			
			while (it.hasNext() ){
				int value = it.next();
				
				LogManager.print(Log.DEBUG, Constants.debugTag,"---Graph: ID: "+data.getId()+" "+value+".."+(String) this.ctx.getText(values.get(value)));
				
				this.render.addYTextLabel((double) value, 
						(String) this.ctx.getText(values.get(value)), pos);
				}
			}		
		}
	}

	public void createGraph(Context context,String title) {
		
		GraphicalView finish = this.getView(context, title);
		this.container.removeAllViews();
		this.container.addView(finish);		
	}
	
	
	
	
	public void setData1(DataGraph data1) {
		this.data1 = data1;
	}

	public DataGraph getData1() {
		return data1;
	}

	public void setData2(DataGraph data2) {
		this.data2 = data2;
	}

	public DataGraph getData2() {
		return data2;
	}

}



	
	

