package com.moonapps.commandroid.listeners;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.ArrayList;

import com.moonapps.commandroid.R;
import com.moonapps.commandroid.adapters.TriggerAdapter;
import com.moonapps.commandroid.agents.Metric;
import com.moonapps.commandroid.agents.MetricsManager;
import com.moonapps.commandroid.profiles.Trigger;

import android.app.Dialog;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.SeekBar;
import android.widget.TextView;

public class ValueContListener implements OnClickListener {

	private TriggerAdapter adapter;
	private Context context;
	private ArrayList<Trigger> triggers;
	private int position;
	
	public ValueContListener (Context context, int position, TriggerAdapter adapter, ArrayList<Trigger> triggers){
		this.adapter=adapter;
		this.context=context;
		this.triggers = triggers;
		this.position = position;
	}
	
	@Override
	public void onClick(View v) {
		Trigger trigg = this.triggers.get(position);
		Metric metric = MetricsManager.metrics.get(trigg.getTriggerID());
		
		Dialog yourDialog = new Dialog(this.context);
		LayoutInflater inflater = (LayoutInflater)this.context.getSystemService(this.context.LAYOUT_INFLATER_SERVICE);
		View layout = inflater.inflate(R.layout.dialog_values,  (ViewGroup) yourDialog.findViewById(R.layout.dialog_values));
		yourDialog.setContentView(layout);

		TextView textMin  = (TextView) layout.findViewById(R.id.textmin);
		TextView textMax  = (TextView) layout.findViewById(R.id.textmax);
		TextView textSelected  = (TextView) layout.findViewById(R.id.textselected);
		textMin.setText(""+(int)metric.getMinValue());
		textMax.setText(""+(int)metric.getMaxValue());
		
		double max = metric.getMaxValue();
		double min = metric.getMinValue(); 
		double x = trigg.getValue();
		
		double percentage = ((x-min) / (max-min))*100;		
		textSelected.setText(""+(int)(percentage));

		SeekBar seekBar = (SeekBar) layout.findViewById(R.id.seekbar);
		seekBar.setProgress((int) (percentage));
		SeekBarListener yourSeekBarListener = new SeekBarListener(
				max, min, textSelected);
		
		seekBar.setOnSeekBarChangeListener(yourSeekBarListener);

		Button accept = (Button) layout.findViewById(R.id.buttonok);
		Button cancel = (Button) layout.findViewById(R.id.buttoncancel);
		cancel.setOnClickListener(new SeekBarCancelListener(yourDialog));
		accept.setOnClickListener(new SeekBarAcceptListener(yourSeekBarListener, adapter, trigg, yourDialog));
		
		
		yourDialog.setTitle(this.context.getText(R.string.dialog_selectvalue));
		yourDialog.show();			
			
	}

}
