package com.moonapps.commandroid.profiles;
/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import android.content.Context;
import android.util.Log;

import com.moonapps.commandroid.LogManager;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.agents.MetricsManager;

public class ActionFactory {
	

	static public Action createSTDAction(int id){
		Action result = ActionFactory.createAction(id, 0);
		
		ActionInfo current = MetricsManager.actions.get(id);
		result.setValue((int) current.getStdValue());
		
		
		return result;
	}
	
	
	static public Action createAction(int id, int value){
		Action result = null;
		

		switch (id){
		
		case(Constants.ACTION_VIBRATE_MODE):
			result = new ActionSilentMode(value); break;
		case(Constants.ACTION_VOLUME_MEDIA):
			result = new ActionVolumeMedia(value); break;
		case(Constants.ACTION_VOLUME_ALARM):
			result = new ActionVolumeAlarm(value); break;
		case(Constants.ACTION_VOLUME_RINGTONE):
			result = new ActionVolumeRingtone(value); break;
		case(Constants.ACTION_AIRPLANE_MODE):
			result = new ActionAirplaneMode(value); break;
		case(Constants.ACTION_WIFI):
			result = new ActionWifi(value); break;
		default: 
			result = new Action(id, value);break;
		}
		
		
		ActionInfo current = MetricsManager.actions.get(id);
		
		if (result.getValue()<current.getMinValue() || result.getValue()>current.getMaxValue())
			result.setValue((int) current.getStdValue());
		
		
		
		return result;

	}


	public static void executeNormalAction(Context ctx, Action act) {


		Action tmp = null;
		switch (act.getActionID()){
		case (Constants.ACTION_VIBRATE_MODE): 
			tmp = new ActionSilentMode(act.getValue());
			tmp.execute(ctx);
			break;
		case (Constants.ACTION_VOLUME_ALARM): 
			tmp = new ActionVolumeAlarm(act.getValue());			
			tmp.execute(ctx);
			break;
		case (Constants.ACTION_VOLUME_MEDIA): 
			tmp = new ActionVolumeMedia(act.getValue());
			tmp.execute(ctx);
			break;
		case (Constants.ACTION_VOLUME_RINGTONE): 
			tmp = new ActionVolumeRingtone(act.getValue());
			tmp.execute(ctx);
			break;
		case (Constants.ACTION_AIRPLANE_MODE): 
			tmp = new ActionAirplaneMode(act.getValue());
			tmp.execute(ctx);
			break;
		default:
			LogManager.print(Log.ERROR, Constants.debugTag, "  ActionFactory: This seems a normal action. Can't do anything.");
		}
		
	}
	
	
	
}
