package com.moonapps.commandroid.receivers;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import java.util.HashMap;

import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.agents.Metric;
import com.moonapps.commandroid.agents.MetricsManager;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.os.BatteryManager;
import android.os.Bundle;
import android.widget.ProgressBar;
import android.widget.TextView;

public class ActivityBatteryReceiver extends BroadcastReceiver {

	
	private TextView health;
	private TextView level;
	private TextView plugged;
	private TextView temperature;
	private TextView status;
	private ProgressBar bar;
	private Context context;

	public ActivityBatteryReceiver(Context context, TextView health,TextView level,
			TextView plugged,TextView status,TextView temperature,
			ProgressBar bar) {
		this.level=level;
		this.status=status;
		this.temperature= temperature;
		this.plugged=plugged;
		this.health=health;
		this.bar=bar;
		this.context=context;
	}
	

	@Override
	public void onReceive(Context context, Intent intent) {
		
        Bundle bundle = intent.getExtras();
    	int tmpHealth = bundle.getInt(BatteryManager.EXTRA_HEALTH);
		int tmpLevel = bundle.getInt(BatteryManager.EXTRA_LEVEL);
		int tmpPlugged = bundle.getInt(BatteryManager.EXTRA_PLUGGED);
		int tmpStatus = bundle.getInt(BatteryManager.EXTRA_STATUS);
		double tmpTemperature = bundle.getInt(BatteryManager.EXTRA_TEMPERATURE)/10.0;
		
		HashMap<Integer, Metric> tmp = MetricsManager.metrics;
		this.health.setText(tmp.get(Constants.REQUEST_BATTERY_HEALTH).getValues().get(tmpHealth));
		this.level.setText(""+tmpLevel+"%");
		this.plugged.setText(tmp.get(Constants.REQUEST_BATTERY_PLUGGED).getValues().get(tmpPlugged));
		this.status.setText(tmp.get(Constants.REQUEST_BATTERY_STATUS).getValues().get(tmpStatus));
		this.temperature.setText(""+tmpTemperature+"C");
		
		
		
		this.bar.setProgress(tmpLevel);
		if (tmpStatus==BatteryManager.BATTERY_STATUS_CHARGING){
			this.bar.setSecondaryProgress(tmpLevel);
		}else{
			if ((tmpLevel+5)>100) 
				tmpLevel=95;
			this.bar.setSecondaryProgress(tmpLevel+5);
		}

		
	}

}