package com.moonapps.commandroid.receivers;

/**
 * Copyright (C) 2012 Daniel Pavon Perez
 *  
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *  
 *      http://www.apache.org/licenses/LICENSE-2.0
 *  
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import java.util.ArrayList;


import com.moonapps.commandroid.LogManager;
import com.moonapps.commandroid.agents.Constants;
import com.moonapps.commandroid.graph.BasicData;
import com.moonapps.commandroid.graph.DataGraph;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

public class ActivityDataReceiver extends BroadcastReceiver {

	private DataGraph data1;
	private DataGraph data2;

	
	
	
	public ActivityDataReceiver(DataGraph data1,DataGraph data2){
		this.data1=data1;
		this.data2=data2;

	}
	
	@Override
	public void onReceive(Context ctx, Intent intent) {
		
		
		LogManager.print(Log.DEBUG,Constants.debugTag,"  ActivityReceiver: Intent: "+ intent.getAction()+"\n");
		
		if (intent.getAction().equals(Constants.ACTION_DATARESPONSE)){
			this.processIntent(ctx, intent);
		}
		
	}

	
	private void processIntent(Context ctx, Intent intent) {

		ArrayList<BasicData> temp = intent
				.getParcelableArrayListExtra(Constants.EXTRAKEY_DATA);

		if (temp == null)
			LogManager.print(Log.ERROR, Constants.debugTag, "  ActivityReceiver: DATA NULL.");
		else { 
			
			int dataType = intent.getIntExtra(Constants.EXTRAKEY_DATATYPE,
					Constants.NO_EXTRA_ERROR);
			int graphNumber = intent.getIntExtra(Constants.EXTRAKEY_GRAPHNUMBER,
					Constants.NO_EXTRA_ERROR);
			

			
			switch (graphNumber) {
			case (Constants.NO_EXTRA_ERROR):
				break;
			case (0):
				this.data1.setData(temp);
				this.data1.setId(dataType); //TODO: COMPROBAR
				break;
			case (1):
				this.data2.setData(temp);
				this.data2.setId(dataType);
				break;
			}
			
			LogManager.print(Log.DEBUG,Constants.debugTag, "  ActivityReceiver: Data1: "+ 
					this.data1.getData().size() + " Data2: "
					+ this.data2.getData().size()+ "  Datatype: "+dataType);

		}
		
	}

	
}



